"""
Implements a collection of gene classes

Genes support the following python operators:
    - + - calculates the phenotype resulting from the
      combination of a pair of genes

These genes work via classical Mendelian genetics
"""

import sys
from random import random, randint, uniform, choice
from math import sqrt

from .xmlio import PGXmlMixin

class BaseGene(PGXmlMixin):
    """
    Base class from which all the gene classes are derived.

    You cannot use this class directly, because there are
    some methods that must be overridden.
    """
    # each gene should have an object in
    # which its genotype should be stored
    value = None

    # probability of a mutation occurring
    mutProb = 0.01

    # List of acceptable fields for the factory
    fields = ["value", "mutProb"]

    def __init__(self):

        # if value is not provided, it will be
        # randomly generated
        if self.__class__.value == None:
            self.value = self.randomValue()
        else:
            self.value = self.__class__.value


    def copy(self):
        """
        returns clone of this gene
        """
        cls = self.__class__()
        cls.value = self.value
        return cls


    def __add__(self, other):
        """
        Combines two genes in a gene pair, to produce an effect

        This is used to determine the gene's phenotype

        This default method computes the arithmetic mean
        of the two genes.

        Override as needed

        Must be overridden
        """
        raise Exception("Method __add__ must be overridden")

    def __repr__(self):
        return "<%s:%s>" % (self.__class__.__name__, self.value)

    # def __cmp__(this, other):
    #     return cmp(this.value, other.value)
    #
    def maybeMutate(self):
        if random() < self.mutProb:
            self.mutate()

    def mutate(self):
        """
        Perform a mutation on the gene

        You MUST override this in subclasses
        """
        raise Exception("method 'mutate' not implemented")

    def randomValue(self):
        """
        Generates a plausible random value
        for this gene.

        Must be overridden
        """
        raise Exception("Method 'randomValue' not implemented")

    def xmlDumpSelf(self, doc, parent):
        """
        dump out this gene into parent tag
        """
        genetag = doc.createElement("gene")
        parent.appendChild(genetag)

        self.xmlDumpClass(genetag)

        self.xmlDumpAttribs(genetag)

        # now dump out the value into a text tag
        ttag = doc.createTextNode(str(self.value))

        # and add it to self tag
        genetag.appendChild(ttag)

    def xmlDumpAttribs(self, tag):
        """
        sets attributes of tag
        """
        tag.setAttribute("mutProb", str(self.mutProb))


class ComplexGene(BaseGene):
    """
    A gene whose value is a complex point number
    """
    # amount by which to mutate, will change value
    # by up to +/- this amount
    mutAmtReal = 0.1
    mutAmtImag = 0.1

    # used for random gene creation
    # override in subclasses
    randMin = -1.0
    randMax = 1.0

    # Acceptable fields for factory
    fields = ["value", "mutProb", "mutAmtReal", "mutAmtImag",
              "randMin", "randMax"]

    def __add__(self, other):
        """
        Combines two genes in a gene pair, to produce an effect

        This is used to determine the gene's phenotype

        This class computes the arithmetic mean
        of the two genes' values, so is akin to incomplete
        dominance.

        Override if desired
        """
        return (self.value + other.value) / 2.0
        #return abs(complex(self.value.real, other.value.imag))


    def mutate(self):
        """
        Mutate this gene's value by a random amount
        within the range +/- self.mutAmt

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value += complex(
            uniform(-self.mutAmtReal, self.mutAmtReal),
            uniform(-self.mutAmtImag, self.mutAmtImag)
            )

        # if the gene has wandered outside the alphabet,
        # rein it back in
        real = self.value.real
        imag = self.value.imag

        if real < self.randMin:
            real = self.randMin
        elif real > self.randMax:
            real = self.randMax

        if imag < self.randMin:
            imag = self.randMin
        elif imag > self.randMax:
            imag = self.randMax

        self.value = complex(real, imag)

    def randomValue(self):
        """
        Generates a plausible random value
        for this gene.

        Override as needed
        """
        min = self.randMin
        range = self.randMax - min

        real = uniform(self.randMin, self.randMax)
        imag = uniform(self.randMin, self.randMax)

        return complex(real, imag)


class FloatGene(BaseGene):
    """
    A gene whose value is a floating point number

    Class variables to override:

        - mutAmt - default 0.1 - amount by which to mutate.
          The gene will will move this proportion towards
          its permissible extreme values

        - randMin - default -1.0 - minimum possible value
          for this gene. Mutation will never allow the gene's
          value to be less than this

        - randMax - default 1.0 - maximum possible value
          for this gene. Mutation will never allow the gene's
          value to be greater than this
    """
    # amount by which to mutate, will change value
    # by up to +/- this amount
    mutAmt = 0.1

    # used for random gene creation
    # override in subclasses
    randMin = -1.0
    randMax = 1.0

    # Acceptable fields for factory
    fields = ["value", "mutProb", "mutAmt", "randMin", "randMax"]

    def __add__(self, other):
        """
        Combines two genes in a gene pair, to produce an effect

        This is used to determine the gene's phenotype

        This class computes the arithmetic mean
        of the two genes' values, so is akin to incomplete
        dominance.

        Override if desired
        """
        return (self.value + other.value) / 2.0

    def mutate(self):
        """
        Mutate this gene's value by a random amount
        within the range, which is determined by
        multiplying self.mutAmt by the distance of the
        gene's current value from either endpoint of legal values

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        if random() < 0.5:
            # mutate downwards
            self.value -= uniform(0, self.mutAmt * (self.value-self.randMin))
        else:
            # mutate upwards:
            self.value += uniform(0, self.mutAmt * (self.randMax-self.value))


    def randomValue(self):
        """
        Generates a plausible random value
        for this gene.

        Override as needed
        """
        return uniform(self.randMin, self.randMax)



class FloatGeneRandom(FloatGene):
    """
    Variant of FloatGene where mutation always randomises the value
    """
    def mutate(self):
        """
        Randomise the gene

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value = self.randomValue()


class FloatGeneRandRange(FloatGene):
    def __add__(self, other):
        """
        A variation of float gene where during the mixing a random value
        from within range created by the two genes is selected.
        """
        start = min([self.value, other.value])
        end = max([self.value, other.value])
        return uniform(start, end)


class FloatGeneMax(FloatGene):
    """
    phenotype of this gene is the greater of the values
    in the gene pair
    """
    def __add__(self, other):
        """
        produces phenotype of gene pair, as the greater of this
        and the other gene's values
        """
        return max(self.value, other.value)

class FloatGeneExchange(FloatGene):
    """
    phenotype of this gene is the random of the values
    in the gene pair
    """
    def __add__(self, other):
        """
        produces phenotype of gene pair, as the random of this
        and the other gene's values
        """
        return choice([self.value, other.value])


class IntGene(BaseGene):
    """
    Implements a gene whose values are ints,
    constrained within the randMin,randMax range
    """
    # minimum possible value for gene
    # override in subclasses as needed
    randMin = -sys.maxsize

    # maximum possible value for gene
    # override in subclasses as needed
    randMax = sys.maxsize + 1

    # maximum amount by which gene can mutate
    mutAmt = 1

    # Acceptable fields for factory
    fields = ["value", "mutProb", "mutAmt", "randMin", "randMax"]

    def mutate(self):
        """
        perform gene mutation

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value += randint(-self.mutAmt, self.mutAmt)

        # if the gene has wandered outside the alphabet,
        # rein it back in
        if self.value < self.randMin:
            self.value = self.randMin
        elif self.value > self.randMax:
            self.value = self.randMax

    def randomValue(self):
        """
        return a legal random value for this gene
        which is in the range [self.randMin, self.randMax]
        """
        return randint(self.randMin, self.randMax)

    def __add__(self, other):
        """
        produces the phenotype resulting from combining
        this gene with another gene in the pair

        returns an int value, based on a formula of higher
        numbers dominating
        """
        return max(self.value, other.value)


class IntGeneRandom(IntGene):
    """
    Variant of IntGene where mutation always randomises the value
    """
    def mutate(self):
        """
        Randomise the gene

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value = self.randomValue()


class IntGeneExchange(IntGene):
    def __add__(self, other):
        """
        A variation of int gene where during the mixing a
        random gene is selected instead of max.
        """
        return choice([self.value, other.value])


class IntGeneAverage(IntGene):
    def __add__(self, other):
        """
        A variation of int gene where during the mixing a
        average of two genes is selected.
        """
        return (self.value + other.value) / 2


class IntGeneRandRange(IntGene):
    def __add__(self, other):
        """
        A variation of int gene where during the mixing a random value
        from within range created by the two genes is selected.
        """
        start = min([self.value, other.value])
        end = max([self.value, other.value])
        return randint(start, end)


class CharGene(BaseGene):
    """
    Gene that holds a single ASCII character,
    as a 1-byte string
    """
    # minimum possible value for gene
    # override in subclasses as needed
    randMin = '\x00'

    # maximum possible value for gene
    # override in subclasses as needed
    randMax = '\xff'

    def __repr__(self):
        """
        Returns safely printable value
        """
        return self.value

    def mutate(self):
        """
        perform gene mutation

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value = ord(self.value) + randint(-int(self.mutAmt), int(self.mutAmt))

        # if the gene has wandered outside the alphabet,
        # rein it back in
        if self.value < ord(self.randMin):
            self.value = self.randMin
        elif self.value > ord(self.randMax):
            self.value = self.randMax
        else:
            self.value = chr(self.value)

    def randomValue(self):
        """
        return a legal random value for this gene
        which is in the range [self.randMin, self.randMax]
        """
        return chr(randint(ord(self.randMin), ord(self.randMax)))

    def __add__(self, other):
        """
        produces the phenotype resulting from combining
        this gene with another gene in the pair

        returns an int value, based on a formula of higher
        numbers dominating
        """
        return max(self.value, other.value)


class CharGeneExchange(CharGene):
    def __add__(self, other):
        """
        A variation of char gene where during the mixing a
        average of two genes is selected.
        """
        return choice([self.value, other.value])


class AsciiCharGene(CharGene):
    """
    Specialisation of CharGene that can only
    hold chars in the legal ASCII range

    OBSOLETE/REMOVE: Exactly the same as chargene
    """
    # minimum possible value for gene
    # override in subclasses as needed
    randMin = chr(0)

    # maximum possible value for gene
    # override in subclasses as needed
    randMax = chr(255)

    def __repr__(self):
        """
        still need to str() the value, since the range
        includes control chars
        """
        return self.value

class PrintableCharGene(AsciiCharGene):
    """
    Specialisation of AsciiCharGene that can only
    hold printable chars
    """
    # minimum possible value for gene
    # override in subclasses as needed
    randMin = ' '

    # maximum possible value for gene
    # override in subclasses as needed
    randMax = chr(127)

    def __repr__(self):
        """
        don't need to str() the char, since
        it's already printable
        """
        return self.value

class DiscreteGene(BaseGene):
    """
    Gene type with a fixed set of possible values, typically
    strings

    Mutation behaviour is that the gene's value may
    spontaneously change into one of its alleles
    """
    # this is the set of possible values
    # override in subclasses
    alleles = []

    # the dominant allele - leave as None
    # if gene has incomplete dominance
    dominant = None

    # the co-dominant alleles - leave empty
    # if gene has simple dominance
    codominant = []

    # the recessive allele - leave as None if there's a dominant
    recessive = None

    def mutate(self):
        """
        Change the gene's value into any of the possible alleles,
        subject to mutation probability 'self.mutProb'

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value = self.randomValue()

    def randomValue(self):
        """
        returns a random allele
        """
        return choice(self.alleles)

    def __add__(self, other):
        """
        determines the phenotype, subject to dominance properties

        returns a tuple of effects
        """
        # got simple dominance?
        if self.dominant in (self.value, other.value):
            # yes
            return (self.dominant,)

        # got incomplete dominance?
        elif self.codominant:
            phenotype = []
            for val in self.value, other.value:
                if val in self.codominant and val not in phenotype:
                    phenotype.append(val)

            # apply recessive, if one exists and no codominant genes present
            if not phenotype:
                if self.recessive:
                    phenotype.append(self.recessive)

            # done
            return tuple(phenotype)

        # got recessive?
        elif self.recessive:
            return (self.recessive,)

        # nothing else
        return ()

class BitGene(BaseGene):
    """
    Implements a single-bit gene
    """
    def __add__(self, other):
        """
        Produces the 'phenotype' as xor of gene pair values
        """
        raise Exception("__add__ method not implemented")



    def mutate(self):
        """
        mutates this gene, toggling the bit
        probabilistically

        perform mutation IN-PLACE, ie don't return mutated copy
        """
        self.value ^= 1

    def randomValue(self):
        """
        Returns a legal random (boolean) value
        """
        return choice([0, 1])


class AndBitGene(BitGene):
    """
    Implements a single-bit gene, whose
    phenotype is the AND of each gene in the pair
    """
    def __add__(self, other):
        """
        Produces the 'phenotype' as xor of gene pair values
        """
        return self.value and other.value


class OrBitGene(BitGene):
    """
    Implements a single-bit gene, whose
    phenotype is the OR of each gene in the pair
    """
    def __add__(self, other):
        """
        Produces the 'phenotype' as xor of gene pair values
        """
        return self.value or other.value



class XorBitGene(BitGene):
    """
    Implements a single-bit gene, whose
    phenotype is the exclusive-or of each gene in the pair
    """
    def __add__(self, other):
        """
        Produces the 'phenotype' as xor of gene pair values
        """
        return self.value ^ other.value

##
# Gene factories
# Necessary for config loading.
##

def _new_factory(cls):
    "Creates gene factories"
    def factory(name, **kw):
        "Gene factory"
        for key in kw.keys():
            if key not in cls.fields:
                raise Exception("Tried to create a gene with an invalid field: " + key)
        # return new.classobj(name, (cls,), kw)
        return type(name, (cls,), kw)
    return factory

ComplexGeneFactory  = _new_factory(ComplexGene)
DiscreteGeneFactory = _new_factory(DiscreteGene)

FloatGeneFactory         = _new_factory(FloatGene)
FloatGeneMaxFactory      = _new_factory(FloatGeneMax)
FloatGeneRandomFactory   = _new_factory(FloatGeneRandom)
FloatGeneRandRangeFactory = _new_factory(FloatGeneRandRange)
FloatGeneExchangeFactory = _new_factory(FloatGeneExchange)

IntGeneFactory          = _new_factory(IntGene)
IntGeneExchangeFactory  = _new_factory(IntGeneExchange)
IntGeneAverageFactory   = _new_factory(IntGeneAverage)
IntGeneRandRangeFactory = _new_factory(IntGeneRandRange)

CharGeneFactory          = _new_factory(CharGene)
AsciiCharGeneFactory     = _new_factory(AsciiCharGene)
PrintableCharGeneFactory = _new_factory(PrintableCharGene)


# utility functions

def rndPair(geneclass):
    """
    Returns a gene pair, comprising two random
    instances of the given gene class
    """
    return (geneclass(), geneclass())



"""
Implements genetic programming organisms.
"""

from random import random, randrange, choice
from math import sqrt

from .organism import BaseOrganism

from .xmlio import PGXmlMixin


class TypeDoesNotExist(Exception):
    """Parameters does not allow to construct a tree."""
    pass

class BaseNode:
    """
    Base class for genetic programming nodes
    """
    def calc(self, **vars):
        """
        evaluates this node, plugging vars into
        the nodes
        """
        raise Exception("method 'calc' not implemented")

class FuncNode(BaseNode):
    """
    Node which holds a function and its argument nodes
    """
    def __init__(self, org, depth, name=None, children=None, type_=None):
        """
        creates this func node
        """
        self.org = org

        if org.type and type_:
            options = [x for x in org.funcsList if x[-1][0] == type_]
        else:
            options = org.funcsList

        if not options:
            raise TypeDoesNotExist

        if name == None:
            # randomly choose a func
            name, func, nargs, typed = choice(options)
        else:
            # lookup func in organism
            func, nargs, typed = org.funcsDict[name]

        # and fill in the args, from given, or randomly
        if not children:
            if typed:
                children = [org.genNode(depth+1, typed[1+i]) for i in range(nargs)]
            else:
                children = [org.genNode(depth+1) for i in range(nargs)]

        self.type = org.type and typed[0] or None
        self.argtype = org.type and typed[1:] or []
        self.name = name
        self.func = func
        self.nargs = nargs
        self.children = children

        # Additional type check
        self.check_types()

    def calc_nodes(self):
        "Return number of nodes in equation"
        cnt = [0]
        def dfs(node):
            cnt[0] += 1
            if isinstance(node, FuncNode):
                for child in node.children:
                    dfs(child)

        dfs(self)
        return cnt[0]

    def calc(self, **vars):
        """
        evaluates this node, plugging vars into
        the nodes
        """
        args = []
        for child in self.children:
            #print "FuncNode.calc: child %s dump:" % (
            #    child.__class__,
            #    )
            #child.dump()
            arg = child.calc(**vars)
            #print "child returned %s" % repr(arg)
            args.append(arg)

        #print "FuncNode.calc: name=%s func=%s vars=%s args=%s" % (
        #    self.name,
        #    self.func,
        #    vars,
        #    args
        #    )
        if self.argtype:
            for i, pair in enumerate(zip(self.argtype, self.children)):
                argtype, child = pair
                if argtype != child.type:
                    msg = (
                        "\n"
                        "Genetical programming type error:\n"
                        "  Function '%s' called with arguments: %s\n"
                        "  Expected %s found %s (%s) for function argument %d\n"
                        "  Tree:"
                    )
                    print(msg % (self.name, args, argtype, child.name, child.type, i + 1))
                    self.org.tree.dump(1)
                    print()
                    raise TypeError

        t = self.func(*args)
        #print self.name, args, t

        if self.type and (type(t) != self.type):
            msg = (
                "\n"
                "Genetical programming type error:\n"
                "  Function '%s' returned %s (%r) instead of type %r\n"
            )
            print(msg % (self.name, t, type(t), self.type))
            self.org.tree.dump(1)
            print()
            raise TypeError

        return t

    def dump(self, level=0):
        indents = "  " * level
        #print indents + "func:" + self.name
        print("%s%s" % (indents, self.name))
        for child in self.children:
            child.dump(level+1)

    def check_types(self):
        "Check if types of this function match its arguments"
        if not self.type:
            return

        for childtype, child in zip(self.argtype, self.children):
            if child.type != childtype:
                msg = (
                    "\n"
                    "Genetical programming type error:\n"
                    "  Function '%s' has children not matching it's types\n"
                    "  types: %r\n"
                    "  children: %r\n"
                    "  child types: %r\n"
                )
                print(msg % (self.name,
                             self.argtype,
                             self.children,
                             [c.type for c in self.children]))
                self.org.tree.dump(1)
                print()
                raise TypeError

    def copy(self, doSplit=False):
        """
        Copies this node and recursively its children, returning
        the copy

        if doSplit is true, then
        cuts off a piece of the tree, to support
        the recombination phase of mating with another program

        returns a quadruple:
             - copy - a copy of this node
             - fragment - fragment to be given to mate
             - lst - list within copy tree to which fragment
               from mate should be written
             - idx - index within the lst at which the fragment
               should be written

        if doSplit is false, then the last 3 tuple items will be None
        """
        if not doSplit:
            # easy case - split has already occurred elsewhere
            # within the tree, so just clone the kids without
            # splitting
            clonedChildren = \
                [child.copy() for child in self.children]

            # now ready to instantiate clone
            copy = FuncNode(self.org, 0, self.name, clonedChildren, type_=self.type)
            return copy

        # choose a child of this node that we might split
        childIdx = randrange(0, self.nargs)
        childToSplit = self.children[childIdx]

        # if child is a terminal, we *must* split here.
        # if child is not terminal, randomly choose whether
        # to split here
        if (random() < 0.33
            or isinstance(childToSplit, TerminalNode)):

            # split at this node, and just copy the kids
            clonedChildren = [
                child.copy() for child in self.children
            ]

            # now ready to instantiate clone
            copy = FuncNode(self.org, 0, self.name, clonedChildren, type_=self.type)
            return copy, childToSplit, clonedChildren, childIdx
        else:
            # delegate the split down to selected child
            clonedChildren = []
            for i in range(self.nargs):
                child = self.children[i]
                if (i == childIdx):
                    # chosen child
                    (clonedChild, fragment, lst, idx) = child.copy(True)
                else:
                    # just clone without splitting
                    clonedChild = child.copy()
                clonedChildren.append(clonedChild)

            # now ready to instantiate clone
            copy = FuncNode(self.org, 0, self.name, clonedChildren, type_=self.type)
            return copy, fragment, lst, idx

    def mutate(self, depth):
        """
        randomly mutates either this tree, or a child
        """
        # 2 in 3 chance of mutating a child of this node
        if random() > 0.33:
            child = choice(self.children)
            if not isinstance(child, TerminalNode):
                child.mutate(depth+1)
                return

        # mutate this node - replace one of its children
        mutIdx = randrange(0, self.nargs)
        new_child = self.org.genNode(depth+1, type_=self.children[mutIdx].type)
        self.children[mutIdx] = new_child
        self.check_types()

        #print "mutate: depth=%s" % depth


class TerminalNode(BaseNode):
    """
    Holds a terminal value
    """

class ConstNode(TerminalNode):
    """
    Holds a constant value
    """
    def __init__(self, org, value=None, type_=None):
        """
        """
        self.org = org

        if value == None:
            if type_:
                options = [x for x in org.consts if type(x) == type_]
            else:
                options = org.consts
            if options:
                value = choice(options)
            else:
                raise TypeDoesNotExist

        self.value = value
        self.type = type_ or type(value)
        self.name = str(value)


    def calc(self, **vars):
        """
        evaluates this node, returns value
        """
        # easy
        return self.value

    def dump(self, level=0):
        indents = "  " * level
        #print "%sconst: {%s}" % (indents, self.value)
        print("%s{%s}" % (indents, self.value))

    def copy(self):
        """
        clone this node
        """
        return ConstNode(self.org, self.value, type_=self.type)


class VarNode(TerminalNode):
    """
    Holds a variable
    """
    def __init__(self, org, name=None, type_=None):
        """
        Inits this node as a var placeholder
        """
        self.org = org

        if name == None:
            if org.type and type_:
                options = [x for x in org.vars if org.funcsVars[x] == type_]
            else:
                options = org.vars

            if options:
                name = choice(options)
            else:
                raise TypeDoesNotExist

        self.name = name
        self.type = org.type and org.funcsVars[name] or None

    def calc(self, **vars):
        """
        Calculates val of this node
        """
        val = vars.get(self.name, 0.0)
        #print "VarNode.calc: name=%s val=%s vars=%s" % (
        #    self.name,
        #    val,
        #    vars,
        #    )
        return val

    def dump(self, level=0):

        indents = "  " * level
        #print indents + "var {" + self.name + "}"
        print("%s{%s}" % (indents, self.name))

    def copy(self):
        """
        clone this node
        """
        return VarNode(self.org, self.name, type_=self.type)

class ProgOrganismMetaclass(type):
    """
    A metaclass which analyses class attributes
    of a ProgOrganism subclass, and builds the
    list of functions and terminals
    """
    def __init__(cls, name, bases, data):
        """
        Create the ProgOrganism class object
        """
        # parent constructor
        super(ProgOrganismMetaclass, cls).__init__(name, bases, data)

        # get the funcs, consts and vars class attribs
        funcs = data['funcs']
        consts = data['consts']
        vars = data['vars']

        # process the funcs
        funcsList = []
        funcsDict = {}
        funcsVars = {}
        for name, func in list(funcs.items()):
            try:
                types = func._types
            except:
                types = None
            funcsList.append((name, func, func.__code__.co_argcount, types))
            funcsDict[name] = (func, func.__code__.co_argcount, types)
        if cls.type:
            funcsVars = dict(tuple(vars))
            vars = [x[0] for x in vars]

        cls.vars = vars
        cls.funcsList = funcsList
        cls.funcsDict = funcsDict
        cls.funcsVars = funcsVars

class ProgOrganism(BaseOrganism, metaclass=ProgOrganismMetaclass):
    """
    Implements an organism for genetic programming

    Introspects to discover functions and terminals.

    You should add the folling class attribs:
        - funcs - a dictionary of funcs, names are func
          names, values are callable objects
        - vars - a list of variable names
        - consts - a list of constant values
    """

    funcs = {}
    vars = []
    consts = []
    type = None

    # maximum tree depth when generating randomly
    maxDepth = 4

    # probability of a mutation occurring
    mutProb = 0.01

    def __init__(self, root=None):
        """
        Creates this organism
        """

        # Cache fitness
        self.fitness_cache = None

        if root == None:
            root = self.genNode(type_=self.type)

        self.tree = root

    def mate(self, mate):
        """
        Perform recombination of subtree elements
        """

        # get copy of self, plus fragment and location details
        tries = 0
        while True:
            tries += 1

            if tries > 20:
                print("Warning: Failed to swap trees for", tries, "times. Continuing...")
                return self.copy(), mate.copy()

            # Get copied trees
            ourRootCopy, ourFrag, ourList, ourIdx = self.split()
            mateRootCopy, mateFrag, mateList, mateIdx = mate.split()

            # Can we swap them?
            if mateFrag.type != ourFrag.type:
                continue

            # Swap
            ourList[ourIdx] = mateFrag
            mateList[mateIdx] = ourFrag

            # Early sanity check
            mateRootCopy.check_types()
            ourRootCopy.check_types()
            break

        # and return both progeny
        child1 = self.__class__(ourRootCopy)
        child2 = self.__class__(mateRootCopy)

        return (child1, child2)

    def mutate(self):
        """
        Mutates this organism's node tree

        returns the mutant
        """
        mutant = self.copy()
        mutant.tree.mutate(1)
        return mutant

    def split(self):
        """
        support for recombination, returns a tuple
        with four values:
            - root - a copy of the tree, except for the fragment
              to be swapped
            - subtree - the subtree fragment to be swapped
            - lst - a list within the tree, containing the
              fragment
            - idx - index within the list where mate's fragment
              should be written
        """
        # otherwise, delegate the split down the tree
        copy, subtree, lst, idx = self.tree.copy(True)
        return (copy, subtree, lst, idx)

    def calc_nodes(self):
        "Calculate nodes in equation"
        return self.tree.calc_nodes()

    def copy(self):
        """
        returns a deep copy of this organism
        """
        try:
            return self.__class__(self.tree.copy())
        except:
            print("self.__class__ = %s" % self.__class__)
            raise

    def dump(self, node=None, level=1):
        """
        prints out this organism's node tree
        """
        self.tree.dump(1)

    def genNode(self, depth=1, type_=None):
        """
        Randomly generates a node to build in
        to this organism
        """
        cnt = 0
        while True:
            cnt += 1
            try:
                if depth > 1 and (depth >= self.initDepth or flipCoin()):
                    # not root, and either maxed depth, or 50-50 chance
                    if flipCoin():
                        # choose a var
                        v = VarNode(self, type_=type_)
                    else:
                        v = ConstNode(self, type_=type_)
                    return v
                else:
                    # either root, or not maxed, or 50-50 chance
                    f = FuncNode(self, depth, type_=type_)
                    return f
            except TypeDoesNotExist:
                if cnt > 50:
                    print("Warning, probably an infinite loop")
                    print("  your options does not allow for tree construction")
                continue


    def xmlDumpSelf(self, doc, parent):
        """
        Dumps out this object's contents into an xml tree

        Arguments:
            - doc - an xml.dom.minidom.Document object
            - parent - an xml.dom.minidom.Element parent, being
              the node into which this node should be placed
        """
        raise Exception("method xmlDumpSelf not implemented")

    def fitness(self):
        """
        Return the fitness level of this organism, as a float

        Should return a number from 0.0 to infinity, where
        0.0 means 'perfect'

        Organisms should evolve such that 'fitness' converges
        to zero.

        This method must be overridden

        In your override, you should generate a set of values,
        either deterministically or randomly, and pass each
        value to both .testFunc() and .calculate(), comparing
        the results and using this to calculate the fitness
        """
        raise Exception("Method 'fitness' not implemented")

    def testFunc(self, **kw):
        """
        this is the 'reference function' toward which
        organisms are trying to evolve

        You must override this in your organism subclass
        """
        raise Exception("method 'testFunc' not implemented")

    def calc(self, **vars):
        """
        Executes this program organism, using the given
        keyword parameters

        You shouldn't need to override this
        """
        #print "org.calc: vars=%s" % str(vars)

        return self.tree.calc(**vars)

def flipCoin():
    """
    randomly returns True/False
    """
    return choice((True, False))


def typed(*args):
    def typed_decorator(f):
        f._types = args
        return f
    return typed_decorator



"""
pygene is a library for genetic algorithms in python

It aims to be very simple to use, and suitable for people
new to genetic algorithms.
"""

version = "0.3.0"

__all__ = [
    'gene', 'gamete', 'organism', 'population', 'xmlio', 'prog',
    'config'
    ]




"""
pygene/population.py - Represents a population of organisms
"""

import random
from random import randrange, choice
from math import sqrt

from .organism import Organism, BaseOrganism

from .xmlio import PGXmlMixin

class Population(PGXmlMixin):
    """
    Represents a population of organisms

    You might want to subclass this

    Overridable class variables:

        - species - Organism class or subclass, being the 'species'
          of organism comprising this population

        - initPopulation - size of population to randomly create
          if no organisms are passed in to constructor

        - childCull - cull to this many children after each generation

        - childCount - number of children to create after each generation

        - incest - max number of best parents to mix amongst the
          kids for next generation, default 10

        - numNewOrganisms - number of random new orgs to add each
          generation, default 0

        - initPopulation - initial population size, default 10

        - mutants - default 0.1 - if mutateAfterMating is False,
          then this sets the percentage of mutated versions of
          children to add to the child population; children to mutate
          are selected based on fitness

    Supports the following python operators:

        - + - produces a new population instances, whose members are
          an aggregate of the members of the values being added

        - [] - int subscript - returns the ith fittest member

    """
    # cull to this many children after each generation
    childCull = 20

    # number of children to create after each generation
    childCount = 100

    # max number of best parents to mix amongst the kids for
    # next generation
    incest = 10

    # parameters governing addition of random new organisms
    numNewOrganisms = 0 # number of new orgs to add each generation

    # set to initial population size
    initPopulation = 10

    # set to species of organism
    species = Organism

    # mutate this proportion of organisms
    mutants = 0.1

    # set this to true to mutate all progeny
    mutateAfterMating = True

    def __init__(self, *items, **kw):
        """
        Create a population with zero or more members

        Arguments:
            - any number of arguments and/or sequences of args,
              where each arg is an instance of this population's
              species. If no arguments are given, organisms are
              randomly created and added automatically, according
              to self.initPopulation and self.species

        Keywords:
            - init - size of initial population to randomly create.
              Ignored if 1 or more constructor arguments are given.
              if not given, value comes from self.initPopulation
            - species - species of organism to create and add. If not
              given, value comes from self.species
        """
        self.organisms = []

        if 'species' in kw:
            species = self.species = kw['species']
        else:
            species = self.species

        if 'init' in kw:
            init = self.initPopulation = kw['init']
        else:
            init = self.initPopulation

        if not items:
            for i in range(init):
                self.add(species())

    def add(self, *args):
        """
        Add an organism, or a population of organisms,
        to this population

        You can also pass lists or tuples of organisms and/or
        populations, to any level of nesting
        """
        for arg in args:
            if isinstance(arg, tuple) or isinstance(arg, list):
                # got a list of things, add them one by one
                self.add(*arg)

            if isinstance(arg, BaseOrganism):
                # add single organism
                self.organisms.append(arg)

            elif isinstance(arg, Population):
                # absorb entire population
                self.organisms.extend(arg)
            else:
                raise TypeError(
                    "can only add Organism or Population objects")

        self.sorted = False

    def __add__(self, other):
        """
        Produce a whole new population consisting of an aggregate
        of this population and the other population's members
        """
        return Population(self, other)

    def getRandom(self, items=None):
        """
        randomly select one of the given items
        (or one of this population's members, if items
        not given).

        Favours fitter members
        """
        if items == None:
            items = self.organisms

        nitems = len(items)
        n2items = nitems * nitems

        # pick one parent randomly, favouring fittest
        idx = int(sqrt(randrange(n2items)))
        return items[nitems - idx - 1]

    def gen(self, nfittest=None, nchildren=None):
        """
        Executes a generation of the population.

        This consists of:
            - producing 'nchildren' children, parented by members
              randomly selected with preference for the fittest
            - culling the children to the fittest 'nfittest' members
            - killing off the parents, and replacing them with the
              children

        Read the source code to study the method of probabilistic
        selection.
        """
        if not nfittest:
            nfittest = self.childCull
        if not nchildren:
            nchildren = self.childCount

        children = []

        # add in some new random organisms, if required
        if self.numNewOrganisms:
            #print "adding %d new organisms" % self.numNewOrganisms
            for i in range(self.numNewOrganisms):
                self.add(self.species())


        # we use square root to skew the selection probability to
        # the fittest

        # get in order, if not already
        self.sort()
        nadults = len(self)

        n2adults = nadults * nadults

        # statistical survey
        #stats = {}
        #for j in xrange(nchildren):
        #    stats[j] = 0

        # wild orgy, have lots of children
        nchildren = 1 if nchildren == 1 else nchildren // 2
        for i in range(nchildren):
            # pick one parent randomly, favouring fittest
            idx1 = idx2 = int(sqrt(randrange(n2adults)))
            parent1 = self[-idx1]

            # pick another parent, distinct from the first parent
            while idx2 == idx1:
                idx2 = int(sqrt(randrange(n2adults)))
            parent2 = self[-idx2]

            #print "picking items %s, %s of %s" % (
            #    nadults - idx1 - 1,
            #    nadults - idx2 - 1,
            #    nadults)

            #stats[nadults - idx1 - 1] += 1
            #stats[nadults - idx2 - 1] += 1

            # get it on, and store the child
            child1, child2 = parent1 + parent2

            # mutate kids if required
            if self.mutateAfterMating:
                child1 = child1.mutate()
                child2 = child2.mutate()

            children.extend([child1, child2])

        # if incestuous, add in best adults
        if self.incest:
            children.extend(self[:self.incest])

        for child in children:
            child.prepare_fitness()

        children.sort()

        # and add in some mutants, a proportion of the children
        # with a bias toward the fittest
        if not self.mutateAfterMating:
            nchildren = len(children)
            n2children = nchildren * nchildren
            mutants = []
            numMutants = int(nchildren * self.mutants)

            # children[0] - fittest
            # children[-1] - worse fitness
            if 0:
                for i in range(numMutants):
                    # pick one parent randomly, favouring fittest
                    idx = int(sqrt(randrange(n2children)))

                    child = children[-idx]
                    mutant = child.mutate()
                    mutant.prepare_fitness()
                    mutants.append(mutant)
            else:
                for i in range(numMutants):
                    mutant = children[i].mutate()
                    mutant.prepare_fitness()
                    mutants.append(mutant)

            children.extend(mutants)
            children.sort()
        #print "added %s mutants" % numMutants
        # sort the children by fitness
        # take the best 'nfittest', make them the new population
        self.organisms[:] = children[:nfittest]

        self.sorted = True

        #return stats
    def __repr__(self):
        """
        crude human-readable dump of population's members
        """
        return str(self.organisms)

    def __getitem__(self, n):
        """
        Return the nth member of this population,
        which we guarantee to be sorted in order from
        fittest first
        """
        self.sort()
        return self.organisms[n]

    def __len__(self):
        """
        return the number of organisms in this population
        """
        return len(self.organisms)

    def fitness(self):
        """
        returns the average fitness value for the population
        """
        fitnesses = [org.get_fitness() for org in self.organisms]

        return sum(fitnesses)/len(fitnesses)

    def best(self):
        """
        returns the fittest member of the population
        """
        self.sort()
        return self[0]

    def sort(self):
        """
        Sorts this population in order of fitness, with
        the fittest first.

        We keep track of whether this population is in order
        of fitness, so we don't perform unnecessary and
        costly sorting
        """
        if not self.sorted:
            for organism in self.organisms:
                organism.prepare_fitness()
            self.organisms.sort()
            self.sorted = True

    # methods for loading/saving to/from xml

    def xmlDumpSelf(self, doc, parent):
        """
        Writes out the contents of this population
        into the xml tree
        """
        # create population element
        pop = doc.createElement("population")
        parent.appendChild(pop)

        # set population class details
        pop.setAttribute("class", self.__class__.__name__)
        pop.setAttribute("module", self.__class__.__module__)

        # set population params as xml tag attributes
        pop.setAttribute("childCull", str(self.childCull))
        pop.setAttribute("childCount", str(self.childCount))

        # dump out organisms
        for org in self.organisms:
            org.xmlDumpSelf(doc, pop)



"""
Implements classes for entire organisms

Organisms produce Gametes (think sperm/egg) via
the .split() method.

Organisms can be mated by the '+' operator, which
produces a child organism.

Subclasses of Organism must override the following methods:
    - fitness - returns a float value representing the
      organism's fitness - a value from 0.0 to infinity, where
      lower is better

Refer to module pygene.prog for organism classes for genetic
programming.
"""

from random import random, randrange, randint, choice

from .gene import BaseGene, rndPair
from .gamete import Gamete

from .xmlio import PGXmlMixin

class BaseOrganism(PGXmlMixin):
    """
    Base class for genetic algo and genetic programming
    organisms

    Best not use this directly, but rather use or subclass from
    one of:
        - Organism
        - MendelOrganism
        - ProgOrganism
    """
    def __add__(self, partner):
        """
        Allows '+' operator for sexual reproduction

        Returns a whole new organism object, whose
        gene pair for each gene name are taken as one
        gene randomly selected from each parent
        """
        return self.mate(partner)

    def mate(self, partner):
        """
        Mates this organism with another organism to
        produce an entirely new organism

        Override this in subclasses
        """
        raise Exception("method 'mate' not implemented")

    def fitness(self):
        """
        Return the fitness level of this organism, as a float.
        Usually instead of this method a caching method 'get_fitness'
        is used, which calls this method always only once on an
        organism.

        Should return a number from 0.0 to infinity, where
        0.0 means 'perfect'

        Organisms should evolve such that 'fitness' converges
        to zero.

        This method must be overridden
        """
        raise Exception("Method 'fitness' not implemented")

    def prepare_fitness(self):
        """
        Is called on all organisms before asking them for their
        fitness. This allows to calculate fitness using a parallel
        processing which is started by prepare_fitness, and finalized
        in 'fitness' method. By default this method does nothing.

        Organisms using this method should usually take care to call
        it themselves in case it wasn't called before hand.
        """
        pass

    def get_fitness(self):
        """
        Return fitness from the cache, and if needed - calculate it.
        """
        if self.fitness_cache is not None:
            return self.fitness_cache
        else:
            self.fitness_cache = self.fitness()
            return self.fitness_cache

    def duel(self, opponent):
        """
        Duels this organism against an opponent

        Returns -1 if this organism loses, 0 if it's
        a tie, or 1 if this organism wins
        """
        #print "BaseOrganism.duel: opponent=%s" % str(opponent)
        a = self.get_fitness()
        b = opponent.get_fitness()
        if a > b:
            return -1
        elif b > a:
            return 1
        else:
            return 0

    def __lt__(self, other):
        """
        Convenience method which invokes duel

        Allows lists of organisms to be sorted
        """
        return self.get_fitness() < other.get_fitness()

    def __lte__(self, other):
        """
        Convenience method which invokes duel

        Allows lists of organisms to be sorted
        """
        return self.get_fitness() <= other.get_fitness()

    def __repr__(self):
        """
        Delivers a minimal string representation
        of this organism.

        Override if needed
        """
        return "<%s:%s>" % (self.__class__.__name__, self.get_fitness())

    def mutate(self):
        """
        Implement the mutation phase

        Must be overridden
        """
        raise Exception("method 'mutate' not implemented")

    def dump(self):
        """
        Produce a detailed human-readable report on
        this organism and its structure
        """
        raise Exception("method 'dump' not implemented")

    def xmlDumpSelf(self, doc, parent):
        """
        Dumps out this object's contents into an xml tree

        Arguments:
            - doc - an xml.dom.minidom.Document object
            - parent - an xml.dom.minidom.Element parent, being
              the node into which this node should be placed
        """
        raise Exception("method xmlDumpSelf not implemented")

    def xmlDumpAttribs(self, elem):
        """
        Dump out the custom attributes of this
        organism

        elem is an xml.dom.minidom.element object
        """

class Organism(BaseOrganism):
    """
    Simple genetic algorithms organism

    Contains only single genes, not pairs (ie, single-helix)

    Note - all organisms are hermaphrodites, which
    can reproduce by mating with another.
    In this implementation, there is no gender.

    Class variables (to override) are:
        - genome - a dict mapping gene names to gene classes

        - mutateOneOnly - default False - dictates whether mutation
          affects one randomly chosen gene unconditionally, or
          all genes subject to the genes' individual mutation settings

        - crossoverRate - default .5 - proportion of genes to
          split out to first child in each pair resulting from
          a mating

    Python operators supported:
        - + - mates two organism instances together, producing a child
        - [] - returns the value of the gene of a given name
        - <, <=, >, >= - compares fitness value to that of another instance
    """
    # dict which maps genotype names to gene classes
    genome = {}

    # dictates whether mutation affects one randomly chosen
    # gene unconditionally, or all genes subject to the genes'
    # own mutation settings

    mutateOneOnly = False

    # proportion of genes to split out to first
    # child
    crossoverRate = 0.5

    def __init__(self, **kw):
        """
        Initialises this organism randomly,
        or from a set of named gene keywords

        Arguments:
            - gamete1, gamete2 - a pair of gametes from which
              to take the genes comprising the new organism.
              May be omitted.

        Keywords:
            - keyword names are gene names within the organism's
              genome, and values are either:
                  - instances of a Gene subclass, or
                  - a Gene subclass (in which case the class will
                    be instantiated to form a random gene object)

        Any gene names in the genome, which aren't given in the
        constructor keywords, will be added as random instances
        of the respective gene class. (Recall that all Gene subclasses
        can be instantiated with no arguments to create a random
        valued gene).
        """
        # the set of genes which comprise this organism
        self.genes = {}

        # Cache fitness
        self.fitness_cache = None

        # remember the gene count
        self.numgenes = len(self.genome)

        # we're being fed a set of zero or more genes
        for name, cls in list(self.genome.items()):

            # set genepair from given arg, or default to a
            # new random instance of the gene
            gene = kw.get(name, cls)

            # if we're handed a gene class instead of a gene object
            # we need to instantiate the gene class
            # to form the needed gene object
            if type(gene) == type and issubclass(gene, BaseGene):
                gene = gene()
            elif not isinstance(gene, BaseGene):
                # If it wasn't a subclass check if it's an instance
                raise Exception(
                    "object given as gene %s %s is not a gene" % (
                        name, repr(gene)))

            # all good - add in the gene to our genotype
            self.genes[name] = gene

    def copy(self):
        """
        returns a deep copy of this organism
        """
        genes = {}
        for name, gene in list(self.genes.items()):
            genes[name] = gene.copy()
        return self.__class__(**genes)

    def mate(self, partner):
        """
        Mates this organism with another organism to
        produce two entirely new organisms via random choice
        of genes from this or the partner
        """
        genotype1 = {}
        genotype2 = {}

        # gene by gene, we assign our and partner's genes randomly
        for name, cls in list(self.genome.items()):

            ourGene = self.genes.get(name, None)
            if not ourGene:
                ourGene = cls()

            partnerGene = partner.genes.get(name, None)
            if not partnerGene:
                partnerGene = cls()

            # randomly assign genes to first or second child
            if random() < self.crossoverRate:
                genotype1[name] = ourGene
                genotype2[name] = partnerGene
            else:
                genotype1[name] = partnerGene
                genotype2[name] = ourGene

        # got the genotypes, now create the child organisms
        child1 = self.__class__(**genotype1)
        child2 = self.__class__(**genotype2)

        # done
        return (child1, child2)

    def __getitem__(self, item):
        """
        allows shorthand for querying the phenotype
        of this organism
        """
        return self.genes[item].value

    def phenotype(self, geneName=None):
        """
        Returns the phenotype resulting from a
        given gene, OR the total phenotype resulting
        from all the genes

        tries to invoke a child class' method
        called 'phen_<name>'
        """
        # if no gene name specified, build up an entire
        # phenotype dict
        if geneName == None:
            phenotype = {}
            for name, cls in list(self.genome.items()):
                val = self.phenotype(name)
                if name not in phenotype:
                    phenotype[name] = []
                phenotype[name].append(val)

            # got the whole phenotype now
            return phenotype

        # just getting the phenotype for one gene pair
        return self.genes[geneName]

    def mutate(self):
        """
        Implement the mutation phase, invoking
        the stochastic mutation method on each
        component gene

        Does not affect this organism, but returns a mutated
        copy of it
        """
        mutant = self.copy()

        if self.mutateOneOnly:
            # unconditionally mutate just one gene
            gene = choice(list(mutant.genes.values()))
            gene.mutate()

        else:
            # conditionally mutate all genes
            for gene in list(mutant.genes.values()):
                gene.maybeMutate()

        return mutant

    def dump(self):
        """
        Produce a detailed human-readable report on
        this organism, its genotype and phenotype
        """
        print("Organism %s:" % self.__class__.__name__)

        print("  Fitness: %s" % self.get_fitness())
        for k,v in list(self.genes.items()):
            print("  Gene: %s = %s" % (k, v))

    def xmlDumpSelf(self, doc, parent):
        """
        Dumps out this object's contents into an xml tree

        Arguments:
            - doc - an xml.dom.minidom.Document object
            - parent - an xml.dom.minidom.Element parent, being
              the node into which this node should be placed
        """
        orgtag = doc.createElement("organism")
        parent.appendChild(orgtag)

        self.xmlDumpClass(orgtag)

        self.xmlDumpAttribs(orgtag)

        # now dump out the constituent genes
        for name, cls in list(self.genome.items()):

            # create a named genepair tag to contain genes
            pairtag = doc.createElement("genepair")
            orgtag.appendChild(pairtag)

            pairtag.setAttribute("name", name)

            # now write out genes
            gene = self.genes[name]
            #print "self.genes[%s] = %s" % (
            #    name,
            #    pair.__class__
            #    )
            gene.xmlDumpSelf(doc, pairtag)


class GenomeSplitOrganism(Organism):
    """
    Don't exchange genes at random - like Organism does,
    but split genome in random point and exchange halves.

    This organism can work better in situation where `connected'
    genes are located close to each other on the genome.
    """
    chromosome_intersections = 2

    def mate(self, partner):
        """
        Mates this organism with another organism to
        produce two entirely new organisms via random choice
        of genome intersection and splitting halves.

        Genes are sorted by keys, and the key name is what groups
        genes in this process.
        """
        genotype1 = {}
        genotype2 = {}

        # 0 1 2 3 4 5
        # G.G.G.G.G.G
        # g.g.g.g.g.g
        #  ^ - split on 0
        # g.g.g.g.g.g
        #           ^-  split in 5:
        # G.G.G.G.G.G

        # Generate two random intersections
        intersections = set(randrange(0, len(self.genome))
                            for i in range(self.chromosome_intersections))

        intersections = list(sorted(intersections))

        source_a = self.genes
        source_b = partner.genes
        # gene by gene, we assign our and partner's genes
        for i, name in enumerate(sorted(self.genome.keys())):
            if i in intersections:
                source_a, source_b = source_b, source_a

            gene_a = source_a.get(name, None)
            if not gene_a:
                gene_a = self.genome[name]()

            gene_b = source_b.get(name, None)
            if not gene_b:
                gene_b = self.genome[name]()

            # assign genes to first or second child
            genotype1[name] = gene_a
            genotype2[name] = gene_b

        # got the genotypes, now create the child organisms
        child1 = self.__class__(**genotype1)
        child2 = self.__class__(**genotype2)

        return (child1, child2)

class MendelOrganism(BaseOrganism):
    """
    Classical Mendelian genetic organism

    Contains a pair of genes for each gene in the genome

    Organisms contain a set of pairs of genes, where the
    genes of each pair must be of the same type.

    Class variables (to override) are:
        - genome - a dict mapping gene names to gene classes

        - mutateOneOnly - default False - if set, then the
          mutation phase will mutate exactly one of the genepairs
          in the genotype, randomly selected. If False, then
          apply mutation to all genes, subject to individual genes'
          mutation settings

    Python operators supported:
        - + - mates two organism instances together, producing a child
        - [] - returns the phenotype produced by the gene pair of a given name
        - <, <=, >, >= - compares fitness value to that of another instance
    """
    # dict which maps genotype names to gene classes
    genome = {}

    # dictates whether mutation affects one randomly chosen
    # gene unconditionally, or all genes subject to the genes'
    # own mutation settings

    mutateOneOnly = False

    def __init__(self, gamete1=None, gamete2=None, **kw):
        """
        Initialises this organism from either two gametes,
        or from a set of named gene keywords

        Arguments:
            - gamete1, gamete2 - a pair of gametes from which
              to take the genes comprising the new organism.
              May be omitted.

        Keywords:
            - keyword names are gene names within the organism's
              genome, and values are either:
                  - a tuple containing two instances of a Gene
                    subclass, or
                  - a Gene subclass (in which case the class will
                    be instantiated twice to form a random gene pair)

        Any gene names in the genome, which aren't given in the
        constructor keywords, will be added as random instances
        of the respective gene class. (Recall that all Gene subclasses
        can be instantiated with no arguments to create a random
        valued gene).
        """
        # the set of genes which comprise this organism
        self.genes = {}

        # Cache fitness
        self.fitness_cache = None

        # remember the gene count
        self.numgenes = len(self.genome)

        if gamete1 and gamete2:
            # create this organism from sexual reproduction
            for name, cls in list(self.genome.items()):
                self.genes[name] = (
                    gamete1[name].copy(),
                    gamete2[name].copy(),
                    )

            # and apply mutation
            #self.mutate()

            # done, easy as that
            return

        # other case - we're being fed a set of zero or more genes
        for name, cls in list(self.genome.items()):

            # set genepair from given arg, or default to a
            # new random instance of the gene
            genepair = kw.get(name, cls)

            # if we're handed a gene class instead of a tuple
            # of 2 genes, we need to instantiate the gene class
            # to form the needed tuple

            if type(genepair) == type and issubclass(genepair, BaseGene):
                genepair = rndPair(genepair)
            else:
                # we're given a tuple; validate the gene pair
                try:
                    gene1, gene2 = genepair
                except:
                    raise TypeError(
                        "constructor keyword values must be tuple of 2 Genes")

                if not isinstance(gene1, BaseGene):
                    raise Exception(
                        "object %s is not a gene" % repr(gene1))

                if not isinstance(gene2, BaseGene):
                    raise Exception(
                        "object %s is not a gene" % repr(gene2))

            # all good - add in the gene pair to our genotype
            self.genes[name] = genepair

    def copy(self):
        """
        returns a deep copy of this organism
        """
        genes = {}
        for name, genepair in list(self.genes.items()):
            genes[name] = (genepair[0].copy(), genepair[1].copy())
        return self.__class__(**genes)

    def split(self):
        """
        Produces a Gamete object from random
        splitting of component gene pairs
        """
        genes1 = {}
        genes2 = {}

        for name, cls in list(self.genome.items()):

            # fetch the pair of genes of that name
            genepair = self.genes[name]

            if randrange(0,2):
                genes1[name] = genepair[0]
                genes2[name] = genepair[1]
            else:
                genes1[name] = genepair[1]
                genes2[name] = genepair[0]

            # and pick one randomly
            #genes[name] = choice(genepair)

        gamete1 = Gamete(self.__class__, **genes1)
        gamete2 = Gamete(self.__class__, **genes2)

        return (gamete1, gamete2)

    def mate(self, partner):
        """
        Mates this organism with another organism to
        produce two entirely new organisms via mendelian crossover
        """
        #return self.split() + partner.split()
        ourGametes = self.split()
        partnerGametes = partner.split()
        child1 = self.__class__(ourGametes[0], partnerGametes[1])
        child2 = self.__class__(ourGametes[1], partnerGametes[0])
        return (child1, child2)

    def __getitem__(self, item):
        """
        allows shorthand for querying the phenotype
        of this organism
        """
        return self.phenotype(item)

    def phenotype(self, geneName=None):
        """
        Returns the phenotype resulting from a
        given gene, OR the total phenotype resulting
        from all the genes

        tries to invoke a child class' method
        called 'phen_<name>'
        """
        # if no gene name specified, build up an entire
        # phenotype dict
        if geneName == None:
            phenotype = {}
            for name, cls in list(self.genome.items()):
                val = self.phenotype(name)
                if name not in phenotype:
                    phenotype[name] = []
                phenotype[name].append(val)

            # got the whole phenotype now
            return phenotype

        # just getting the phenotype for one gene pair

        if not isinstance(geneName, str):
            geneName = str(geneName)

        try:
            #return sum(self.genes[geneName])
            genes = self.genes[geneName]
            return genes[0] + genes[1]
        except:
            #print "self.genes[%s] = %s" % (geneName, self.genes[geneName])
            raise

        # FIXME: There's an error here for sure. The code is unreachable
        # Maybe it is supposed to be turned off.

        # get the genes in question
        gene1, gene2 = self.genes[geneName]

        # try to find a specialised phenotype
        # calculation method
        methname = 'phen_' + geneName
        meth = getattr(self, methname, None)

        if meth:
            # got the method - invoke it
            return meth(gene1, gene2)
        else:
            # no specialised methods, apply the genes'
            # combination methods
            return gene1 + gene2

    def mutate(self):
        """
        Implement the mutation phase, invoking
        the stochastic mutation method on each
        component gene

        Does not affect this organism, but returns a mutated
        copy of it
        """
        mutant = self.copy()

        if self.mutateOneOnly:
            # unconditionally mutate just one gene
            genepair = choice(list(mutant.genes.values()))
            genepair[0].mutate()
            genepair[1].mutate()

        else:
            # conditionally mutate all genes
            for gene_a, gene_b in list(mutant.genes.values()):
                gene_a.maybeMutate()
                gene_b.maybeMutate()

        return mutant

    def dump(self):
        """
        Produce a detailed human-readable report on
        this organism, its genotype and phenotype
        """
        print("Organism %s:" % self.__class__.__name__)

        print("  Fitness: %s" % self.get_fitness())
        for k,v in list(self.genes.items()):
            print("  Gene: %s" % k)
            print("    Phenotype: %s" % self[k])
            print("    Genotype:")
            print("      %s" % v[0])
            print("      %s" % v[1])

    def xmlDumpSelf(self, doc, parent):
        """
        Dumps out this object's contents into an xml tree

        Arguments:
            - doc - an xml.dom.minidom.Document object
            - parent - an xml.dom.minidom.Element parent, being
              the node into which this node should be placed
        """
        orgtag = doc.createElement("organism")
        parent.appendChild(orgtag)

        self.xmlDumpClass(orgtag)

        self.xmlDumpAttribs(orgtag)

        # now dump out the constituent genes
        for name, cls in list(self.genome.items()):

            # create a named genepair tag to contain genes
            pairtag = doc.createElement("genepair")
            orgtag.appendChild(pairtag)

            pairtag.setAttribute("name", name)

            # now write out genes
            pair = self.genes[name]
            #print "self.genes[%s] = %s" % (
            #    name,
            #    pair.__class__
            #    )
            for gene in pair:
                gene.xmlDumpSelf(doc, pairtag)



"""
Implements gametes, which are the result of
splitting an organism's genome in two, and are
used in the organism's sexual reproduction

In our model, I don't use any concept of a chromosome.
In biology, during a cell's interphase, there are
no chromosomes as such - the genetic material
is scattered chaotically throughout the cell nucleus.

Chromosomes (from my limited knowledge of biologi)
are mostly just a device used in cell division.
Since division of cells in this model isn't
constrained by the physical structure of the cell,
we shouldn't need a construct of chromosomes.

Gametes support the python '+' operator for sexual
reproduction. Adding two gametes together produces
a whole new Organism.
"""

from .xmlio import PGXmlMixin

class Gamete(PGXmlMixin):
    """
    Contains a set of genes.

    Two gametes can be added together to form a
    new organism
    """
    def __init__(self, orgclass, **genes):
        """
        Creates a new gamete from a set of genes
        """
        self.orgclass = orgclass
        self.genes = dict(genes)

    def __getitem__(self, name):
        """
        Fetch a single gene by name
        """
        return self.genes[name]

    def __add__(self, other):
        """
        Combines this gamete with another
        gamete to form an organism
        """
        return self.conceive(other)

    def conceive(self, other):
        """
        Returns a whole new Organism class
        from the combination of this gamete with another
        """
        if not isinstance(other, Gamete):
            raise Exception("Trying to mate a gamete with a non-gamete")

        return self.orgclass(self, other)



"""
Parse genome from config file.
Only int and float genes are supported currently.

Example config file:
[x1]
type = float
randMin = -100.0
randMax = 100.0
mutProb = 0.1
mutAmt = 0.1

[x2]
type = int
randMin = -50
randMax = 50
mutProb = 0.2
mutAmt = 1

One section per gene.
'type' is necessary - other fields depends on the selected type
"""

import configparser
from configparser import NoOptionError

from .gene import ComplexGeneFactory
from .gene import IntGeneFactory, IntGeneExchangeFactory
from .gene import IntGeneAverageFactory, IntGeneRandRangeFactory
from .gene import FloatGeneFactory, FloatGeneRandomFactory, FloatGeneMaxFactory
from .gene import FloatGeneExchangeFactory, FloatGeneRandRangeFactory

class LoaderError(Exception):
    pass

# Casts to int and float (TODO: Add other)
def _intcast(section, key, value):
    "Parse string into int or None with correct exceptions"
    if not value: # '' and None are coerced to None
        return None

    try:
        return int(value)
    except ValueError:
        raise LoaderError("Invalid integer value '%s' in position %s / %s." % (
                          value, section, key))

def _floatcast(section, key, value):
    "Parse string into float or None with correct exceptions"
    if not value:
        return None
    try:
        return float(value)
    except ValueError:
        raise LoaderError("Invalid floating-point value '%s' in position %s / %s." % (
                          value, section, key))

class ConfigLoader(object):

    def __init__(self, filename, require_genes=[], config_contents=None):
        """
        Genome loader.
        Filename - path to configuration (alternatively you can pass the config
        contents via the config_contents and pass None as the filename).
        If require_genes are passed after the loading we ensure that
        they exist.
        """
        # Dictionary of supported types into casts and factories
        self.types = {
            'int': (_intcast, IntGeneFactory),
            'int_exchange': (_intcast, IntGeneExchangeFactory),
            'int_average': (_intcast, IntGeneAverageFactory),
            'int_randrange': (_intcast, IntGeneRandRangeFactory),

            'float': (_floatcast, FloatGeneFactory),
            'float_average': (_floatcast, FloatGeneFactory), # The same as float.
            'float_randrange': (_floatcast, FloatGeneRandRangeFactory),
            'float_exchange': (_floatcast, FloatGeneExchangeFactory),
            'float_random': (_floatcast, FloatGeneRandomFactory),
            'float_max': (_floatcast, FloatGeneMaxFactory),

            'complex': (_floatcast, ComplexGeneFactory),
            # 'char': (str, CharGeneFactory),
            # ... TODO: add other
        }

        self.genome = {}

        self.require_genes = require_genes

        self.config = configparser.RawConfigParser()
        self.config.optionxform = str # Don't lower() names

        if filename is None and config_contents is not None:
            import io
            self.config.readfp(io.BytesIO(config_contents))
        else:
            self.config.read(filename)

        # Do we have a population definition also?
        self._pre_parse_population()


    def register_type(self, typename, cast, factory):
        """
        User can register his types using this method
        """
        self.types[typename] = (cast, factory)

    def _pre_parse_population(self):
        self.has_population = self.config.has_section('population')
        try:
            genes = self.config.get('population', 'genes')
            genes = [gene.strip()
                     for gene in genes.split()]
            self.genes = genes if genes else None
        except NoOptionError:
            self.genes = None

    def load_population(self, name, species):
        """
        Parse population options and return a population
        """
        from .population import Population

        if not self.has_population:
            raise LoaderError("No population is defined in the config file")
        args = {
            'species': species
        }
        def parse(fun, name):
            if self.config.has_option('population', name):
                try:
                    val = fun('population', name)
                    args[name] = val
                except ValueError:
                    raise LoaderError("Invalid value for population option " + name)

        parse(self.config.getint, 'initPopulation')
        parse(self.config.getint, 'childCull')
        parse(self.config.getint, 'childCount')
        parse(self.config.getint, 'incest')
        parse(self.config.getint, 'numNewOrganisms')
        parse(self.config.getboolean, 'mutateAfterMating')
        parse(self.config.getfloat, 'mutants')

        return type(name, (Population,), args)


    def _parse_gene(self, section):
        """
        parse_gene is called by parse_genome to parse
        a single gene from a config.
        config - ConfigParser instance
        section - gene name / config section name
        """
        genename = section

        if not self.config.has_section(genename):
            raise LoaderError("Gene %s has no section in the config file" % genename)

        # FIXME: This check won't work because of configparser:
        if genename in self.genome:
            raise LoaderError("Gene %s was already defined" % section)

        try:
            clonegene = self.config.get(section, 'clone')
            if not self.config.has_section(clonegene):
                raise LoaderError("Gene %s is cloning a gene %s which is not yet defined" % (genename, clonegene))
            section = clonegene
            genename = clonegene
        except NoOptionError:
            pass

        try:
            typename = self.config.get(section, 'type')
        except NoOptionError:
            raise LoaderError(("Required field 'type' was not "
                              "found in section/gene '%s'") % section)

        try:
            cast, factory = self.types[typename]
        except KeyError:
            raise LoaderError("Unhandled type in config file: " + typename)

        args = {}
        for key, value in self.config.items(section):
            if key == "type":
                continue
            if key == "mutProb": # Always float
                converted = _floatcast(section, key, value)
            else:
                converted = cast(section, key, value)
            args[key] = converted

        if 'randMin' in args and 'randMax' in args:
            if args['randMin'] > args['randMax']:
                raise LoaderError('randMin higher than randMax in section/gene %s' % section)
            if ('value' in args and args['value'] is not None
                and (args['value'] > args['randMax']
                     or args['value'] < args['randMin'])):
                raise LoaderError('value not within randMin, randMax in section/gene %s' % section)

        gene = factory(typename + "_" + genename, **args)
        return gene

    def load_genome(self):
        """
        Load genome from config file
        """

        sections = self.genes if self.genes else self.config.sections()

        for section in sections:
            if section.lower() == 'population':
                continue
            elif self.config.has_option(section, 'alias'):
                alias = self.config.get(section, 'alias')
                if alias not in self.genome:
                    raise LoaderError(("Gene %s is an alias for non-existing gene %s. "
                                       "Order matters!") % (section, alias))
                self.genome[section] = self.genome[alias]
                continue
            else:
                gene = self._parse_gene(section)
                self.genome[section] = gene

        for gene in self.require_genes:
            if gene not in self.genome:
                raise LoaderError("Required gene '%s' was not found in the config" % gene)
        #for gene in self.genome.itervalues():
        #    print gene.__dict__
        return self.genome



"""
xmlio.py

Mixin class to support pygene objects in
loading/saving as xml
"""

import io
from xml.dom.minidom import getDOMImplementation, parse, parseString

domimpl = getDOMImplementation()

class PGXmlMixin(object):
    """
    mixin class to support pygene classes
    serialising themselves to/from xml
    """
    def xmlDump(self, fileobj):
        """
        Dumps out the population to an open file in XML format.

        To dump to a string, use .xmlDumps() instead
        """
        doc = domimpl.createDocument(None, "pygene", None)

        top = doc.documentElement
        top.appendChild(doc.createComment(
            "generated by pygene - http://www.freenet.org.nz/python/pygene"))

        self.xmlDumpSelf(doc, top)

        fileobj.write(doc.toxml())




    def xmlDumps(self):
        """
        dumps out to xml, returning a string of the raw
        generated xml
        """
        s = io.StringIO()
        self.xmlDump(s)
        return s.getvalue()

    def xmlDumpSelf(self, doc, parent):
        """
        Writes out the contents of this population
        into the xml tree
        """
        raise Exception("class %s: xmlDumpSelf not implemented" % \
            self.__class__.__name__)

    def xmlDumpClass(self, tag):
        """
        dumps out class information
        """
        tag.setAttribute("class", self.__class__.__name__)
        tag.setAttribute("module", self.__class__.__module__)

    def xmlDumpAttribs(self, tag):
        """
        """
